import datetime
from files_sdk.api import Api
from files_sdk.exceptions import InvalidParameterError, MissingParameterError, NotImplementedError

class Preview:
    default_attributes = {
        'id': None,     # int64 - Preview ID
        'status': None,     # string - Preview status.  Can be invalid, not_generated, generating, complete, or file_too_large
        'download_uri': None,     # string - Link to download preview
        'type': None,     # string - Preview status.  Can be invalid, not_generated, generating, complete, or file_too_large
        'size': None,     # int64 - Preview size
    }

    def __init__(self, attributes=None, options=None):
        if not isinstance(attributes, dict):
            attributes = {}
        if not isinstance(options, dict):
            options = {}
        self.set_attributes(attributes)
        self.options = options

    def set_attributes(self, attributes):
        for (attribute, default_value) in Preview.default_attributes.items():
            setattr(self, attribute, attributes.get(attribute, default_value))

    def get_attributes(self):
        return {k: getattr(self, k, None) for k in Preview.default_attributes if getattr(self, k, None) is not None}


def new(*args, **kwargs):
    return Preview(*args, **kwargs)


import copy
import sys
import files_sdk
from files_sdk.api_client import ApiClient
      
class Api:
    @staticmethod
    def api_client():
        return ApiClient()

    @staticmethod
    def send_request(verb, path, params, options = None):
        if not isinstance(options, dict):
            options = {}
        Api.warn_on_options_in_params(params)

        headers = copy.deepcopy(options)
        api_key = headers.pop("api_key", None)
        session_id = headers.pop("session_id", None)

        session = headers.pop("session", None)
        if session:
            if session.id:
               session.save
            session_id = str(session.id)

        client = ApiClient()

        response = client.send_request(verb, path, api_key=api_key, session_id=session_id, params=params)

        # Remove options not in the allow list
        options = {k: options[k] for k in files_sdk.OPTS if k in options}

        return response, options

        # Hash#select returns an array before 1.9
        #options_to_persist = {}
        #options.each do |k, v|
        #  options_to_persist[k] = v if Util::OPTS.include?(k)
        #end

    @staticmethod
    def warn_on_options_in_params(params):
        for opt in files_sdk.OPTS:
            if opt in params:
                print("WARNING: {opt} should be in the options dictionary, not the params dictionary.  You may need to create a second dictionary that goes after params.)".format(opt=opt), file=sys.stderr)



import datetime
from files_sdk.api import Api
from files_sdk.list_obj import ListObj
from files_sdk.exceptions import InvalidParameterError, MissingParameterError, NotImplementedError

class Notification:
    default_attributes = {
        'id': None,     # int64 - Notification ID
        'path': None,     # string - Folder path to notify on This must be slash-delimited, but it must neither start nor end with a slash. Maximum of 5000 characters.
        'group_id': None,     # int64 - Notification group id
        'group_name': None,     # string - Group name if applicable
        'notify_user_actions': None,     # boolean - Trigger notification on notification user actions?
        'notify_on_copy': None,     # boolean - Triggers notification when moving or copying files to this path
        'recursive': None,     # boolean - Enable notifications for each subfolder in this path
        'send_interval': None,     # string - The time interval that notifications are aggregated to
        'unsubscribed': None,     # boolean - Is the user unsubscribed from this notification?
        'unsubscribed_reason': None,     # string - The reason that the user unsubscribed
        'user_id': None,     # int64 - Notification user ID
        'username': None,     # string - Notification username
        'suppressed_email': None,     # boolean - If true, it means that the recipient at this user's email address has manually unsubscribed from all emails, or had their email "hard bounce", which means that we are unable to send mail to this user's current email address. Notifications will resume if the user changes their email address.
    }

    def __init__(self, attributes=None, options=None):
        if not isinstance(attributes, dict):
            attributes = {}
        if not isinstance(options, dict):
            options = {}
        self.set_attributes(attributes)
        self.options = options

    def set_attributes(self, attributes):
        for (attribute, default_value) in Notification.default_attributes.items():
            setattr(self, attribute, attributes.get(attribute, default_value))

    def get_attributes(self):
        return {k: getattr(self, k, None) for k in Notification.default_attributes if getattr(self, k, None) is not None}

    # Parameters:
    #   notify_on_copy - boolean - If `true`, copying or moving resources into this path will trigger a notification, in addition to just uploads.
    #   notify_user_actions - boolean - If `true` actions initiated by the user will still result in a notification
    #   recursive - boolean - If `true`, enable notifications for each subfolder in this path
    #   send_interval - string - The time interval that notifications are aggregated by.  Can be `five_minutes`, `fifteen_minutes`, `hourly`, or `daily`.
    def update(self, params = None):
        if not isinstance(params, dict):
            params = {}

        if hasattr(self, "id") and self.id:
            params['id'] = self.id
        else:
            raise MissingParameterError("Current object doesn't have a id")
        if "id" not in params:
            raise MissingParameterError("Parameter missing: id")
        if "id" in params and not isinstance(params["id"], int):
            raise InvalidParameterError("Bad parameter: id must be an int")
        if "send_interval" in params and not isinstance(params["send_interval"], str):
            raise InvalidParameterError("Bad parameter: send_interval must be an str")
        response, _options = Api.send_request("PATCH", "/notifications/{id}".format(id=params['id']), params, self.options)
        return response.data

    def delete(self, params = None):
        if not isinstance(params, dict):
            params = {}

        if hasattr(self, "id") and self.id:
            params['id'] = self.id
        else:
            raise MissingParameterError("Current object doesn't have a id")
        if "id" not in params:
            raise MissingParameterError("Parameter missing: id")
        if "id" in params and not isinstance(params["id"], int):
            raise InvalidParameterError("Bad parameter: id must be an int")
        response, _options = Api.send_request("DELETE", "/notifications/{id}".format(id=params['id']), params, self.options)
        return response.data

    def destroy(self, params = None):
        self.delete(params)

    def save(self):
        if hasattr(self, "id") and self.id:
            self.update(self.get_attributes())
        else:
            new_obj = create(self.get_attributes(), self.options)
            self.set_attributes(new_obj.get_attributes())

# Parameters:
#   user_id - int64 - DEPRECATED: Show notifications for this User ID. Use `filter[user_id]` instead.
#   cursor - string - Used for pagination.  Send a cursor value to resume an existing list from the point at which you left off.  Get a cursor from an existing list via the X-Files-Cursor-Next header.
#   per_page - int64 - Number of records to show per page.  (Max: 10,000, 1,000 or less is recommended).
#   sort_by - object - If set, sort records by the specified field in either 'asc' or 'desc' direction (e.g. sort_by[last_login_at]=desc). Valid fields are `path`, `user_id` or `group_id`.
#   filter - object - If set, return records where the specifiied field is equal to the supplied value. Valid fields are `user_id`, `group_id` or `path`.
#   filter_gt - object - If set, return records where the specifiied field is greater than the supplied value. Valid fields are `user_id`, `group_id` or `path`.
#   filter_gteq - object - If set, return records where the specifiied field is greater than or equal to the supplied value. Valid fields are `user_id`, `group_id` or `path`.
#   filter_like - object - If set, return records where the specifiied field is equal to the supplied value. Valid fields are `user_id`, `group_id` or `path`.
#   filter_lt - object - If set, return records where the specifiied field is less than the supplied value. Valid fields are `user_id`, `group_id` or `path`.
#   filter_lteq - object - If set, return records where the specifiied field is less than or equal to the supplied value. Valid fields are `user_id`, `group_id` or `path`.
#   group_id - int64 - DEPRECATED: Show notifications for this Group ID. Use `filter[group_id]` instead.
#   path - string - Show notifications for this Path.
#   include_ancestors - boolean - If `include_ancestors` is `true` and `path` is specified, include notifications for any parent paths. Ignored if `path` is not specified.
def list(params = None, options = None):
    if not isinstance(params, dict):
        params = {}
    if not isinstance(options, dict):
        options = {}
    if "user_id" in params and not isinstance(params["user_id"], int):
        raise InvalidParameterError("Bad parameter: user_id must be an int")
    if "cursor" in params and not isinstance(params["cursor"], str):
        raise InvalidParameterError("Bad parameter: cursor must be an str")
    if "per_page" in params and not isinstance(params["per_page"], int):
        raise InvalidParameterError("Bad parameter: per_page must be an int")
    if "sort_by" in params and not isinstance(params["sort_by"], dict):
        raise InvalidParameterError("Bad parameter: sort_by must be an dict")
    if "filter" in params and not isinstance(params["filter"], dict):
        raise InvalidParameterError("Bad parameter: filter must be an dict")
    if "filter_gt" in params and not isinstance(params["filter_gt"], dict):
        raise InvalidParameterError("Bad parameter: filter_gt must be an dict")
    if "filter_gteq" in params and not isinstance(params["filter_gteq"], dict):
        raise InvalidParameterError("Bad parameter: filter_gteq must be an dict")
    if "filter_like" in params and not isinstance(params["filter_like"], dict):
        raise InvalidParameterError("Bad parameter: filter_like must be an dict")
    if "filter_lt" in params and not isinstance(params["filter_lt"], dict):
        raise InvalidParameterError("Bad parameter: filter_lt must be an dict")
    if "filter_lteq" in params and not isinstance(params["filter_lteq"], dict):
        raise InvalidParameterError("Bad parameter: filter_lteq must be an dict")
    if "group_id" in params and not isinstance(params["group_id"], int):
        raise InvalidParameterError("Bad parameter: group_id must be an int")
    if "path" in params and not isinstance(params["path"], str):
        raise InvalidParameterError("Bad parameter: path must be an str")
    return ListObj(Notification,"GET", "/notifications", params, options)

def all(params = None, options = None):
    list(params, options)

# Parameters:
#   id (required) - int64 - Notification ID.
def find(id, params = None, options = None):
    if not isinstance(params, dict):
        params = {}
    if not isinstance(options, dict):
        options = {}
    params["id"] = id
    if "id" in params and not isinstance(params["id"], int):
        raise InvalidParameterError("Bad parameter: id must be an int")
    if "id" not in params:
        raise MissingParameterError("Parameter missing: id")
    response, options = Api.send_request("GET", "/notifications/{id}".format(id=params['id']), params, options)
    return Notification(response.data, options)

def get(id, params = None, options = None):
    find(id, params, options)

# Parameters:
#   user_id - int64 - The id of the user to notify. Provide `user_id`, `username` or `group_id`.
#   notify_on_copy - boolean - If `true`, copying or moving resources into this path will trigger a notification, in addition to just uploads.
#   notify_user_actions - boolean - If `true` actions initiated by the user will still result in a notification
#   recursive - boolean - If `true`, enable notifications for each subfolder in this path
#   send_interval - string - The time interval that notifications are aggregated by.  Can be `five_minutes`, `fifteen_minutes`, `hourly`, or `daily`.
#   group_id - int64 - The ID of the group to notify.  Provide `user_id`, `username` or `group_id`.
#   path - string - Path
#   username - string - The username of the user to notify.  Provide `user_id`, `username` or `group_id`.
def create(params = None, options = None):
    if not isinstance(params, dict):
        params = {}
    if not isinstance(options, dict):
        options = {}
    if "user_id" in params and not isinstance(params["user_id"], int):
        raise InvalidParameterError("Bad parameter: user_id must be an int")
    if "send_interval" in params and not isinstance(params["send_interval"], str):
        raise InvalidParameterError("Bad parameter: send_interval must be an str")
    if "group_id" in params and not isinstance(params["group_id"], int):
        raise InvalidParameterError("Bad parameter: group_id must be an int")
    if "path" in params and not isinstance(params["path"], str):
        raise InvalidParameterError("Bad parameter: path must be an str")
    if "username" in params and not isinstance(params["username"], str):
        raise InvalidParameterError("Bad parameter: username must be an str")
    response, options = Api.send_request("POST", "/notifications", params, options)
    return Notification(response.data, options)

# Parameters:
#   notify_on_copy - boolean - If `true`, copying or moving resources into this path will trigger a notification, in addition to just uploads.
#   notify_user_actions - boolean - If `true` actions initiated by the user will still result in a notification
#   recursive - boolean - If `true`, enable notifications for each subfolder in this path
#   send_interval - string - The time interval that notifications are aggregated by.  Can be `five_minutes`, `fifteen_minutes`, `hourly`, or `daily`.
def update(id, params = None, options = None):
    if not isinstance(params, dict):
        params = {}
    if not isinstance(options, dict):
        options = {}
    params["id"] = id
    if "id" in params and not isinstance(params["id"], int):
        raise InvalidParameterError("Bad parameter: id must be an int")
    if "send_interval" in params and not isinstance(params["send_interval"], str):
        raise InvalidParameterError("Bad parameter: send_interval must be an str")
    if "id" not in params:
        raise MissingParameterError("Parameter missing: id")
    response, options = Api.send_request("PATCH", "/notifications/{id}".format(id=params['id']), params, options)
    return Notification(response.data, options)

def delete(id, params = None, options = None):
    if not isinstance(params, dict):
        params = {}
    if not isinstance(options, dict):
        options = {}
    params["id"] = id
    if "id" in params and not isinstance(params["id"], int):
        raise InvalidParameterError("Bad parameter: id must be an int")
    if "id" not in params:
        raise MissingParameterError("Parameter missing: id")
    response, _options = Api.send_request("DELETE", "/notifications/{id}".format(id=params['id']), params, options)
    return response.data

def destroy(id, params = None, options = None):
    delete(id, params, options)

def new(*args, **kwargs):
    return Notification(*args, **kwargs)


import datetime
from files_sdk.api import Api
from files_sdk.list_obj import ListObj
from files_sdk.exceptions import InvalidParameterError, MissingParameterError, NotImplementedError

class UsageSnapshot:
    default_attributes = {
        'id': None,     # int64 - Site usage ID
        'start_at': None,     # date-time - Site usage report start date/time
        'end_at': None,     # date-time - Site usage report end date/time
        'created_at': None,     # date-time - Site usage report created at date/time
        'high_water_user_count': None,     # double - Site usage report highest usage in time period
        'current_storage': None,     # double - Current site usage as of report
        'high_water_storage': None,     # double - Site usage report highest usage in time period
        'total_downloads': None,     # int64 - Number of downloads in report time period
        'total_uploads': None,     # int64 - Number of uploads in time period
        'updated_at': None,     # date-time - The last time this site usage report was updated
        'usage_by_top_level_dir': None,     # object - A map of root folders to their total usage
        'root_storage': None,     # double - Usage for root folder
        'deleted_files_counted_in_minimum': None,     # double - Usage for files that are deleted but uploaded within last 30 days
        'deleted_files_storage': None,     # double - Usage for files that are deleted but retained as backups
    }

    def __init__(self, attributes=None, options=None):
        if not isinstance(attributes, dict):
            attributes = {}
        if not isinstance(options, dict):
            options = {}
        self.set_attributes(attributes)
        self.options = options

    def set_attributes(self, attributes):
        for (attribute, default_value) in UsageSnapshot.default_attributes.items():
            setattr(self, attribute, attributes.get(attribute, default_value))

    def get_attributes(self):
        return {k: getattr(self, k, None) for k in UsageSnapshot.default_attributes if getattr(self, k, None) is not None}


# Parameters:
#   cursor - string - Used for pagination.  Send a cursor value to resume an existing list from the point at which you left off.  Get a cursor from an existing list via the X-Files-Cursor-Next header.
#   per_page - int64 - Number of records to show per page.  (Max: 10,000, 1,000 or less is recommended).
def list(params = None, options = None):
    if not isinstance(params, dict):
        params = {}
    if not isinstance(options, dict):
        options = {}
    if "cursor" in params and not isinstance(params["cursor"], str):
        raise InvalidParameterError("Bad parameter: cursor must be an str")
    if "per_page" in params and not isinstance(params["per_page"], int):
        raise InvalidParameterError("Bad parameter: per_page must be an int")
    return ListObj(UsageSnapshot,"GET", "/usage_snapshots", params, options)

def all(params = None, options = None):
    list(params, options)

def new(*args, **kwargs):
    return UsageSnapshot(*args, **kwargs)


class Error(Exception):
    def __init__(
        self,
        message=None,
        http_body=None,
        http_status=None,
        json_body=None,
        headers=None,
        code=None,
    ):
        super(Error, self).__init__(message)

        if http_body and hasattr(http_body, "decode"):
            try:
                http_body = http_body.decode("utf-8")
            except BaseException:
                http_body = (
                    "<Could not decode body as utf-8. "
                )

        self._message = message
        self.http_body = http_body
        self.http_status = http_status
        self.json_body = json_body
        self.headers = headers or {}
        self.code = code
        self.response = None

    def __str__(self):
        msg = ""
        if self.http_status:
            msg += "HTTP {} ".format(self.http_status)
        msg += self._message
        return msg

class APIConnectionError(Error):
    pass

class APIError(Error):
    pass

class AuthenticationError(Error):
    pass

class InvalidRequestError(Error):
    pass

class NotImplementedError(Error):
    pass

class PermissionsError(Error):
    pass

class RateLimitError(Error):
    pass

class TooManyRequestsError(Error):
    pass

class ValidationError(Error):
    pass


class InvalidParameterError(InvalidRequestError):
    pass

class MissingParameterError(InvalidRequestError):
    pass


import datetime
from files_sdk.models.usage_snapshot import UsageSnapshot
from files_sdk.api import Api
from files_sdk.exceptions import InvalidParameterError, MissingParameterError, NotImplementedError

class Site:
    default_attributes = {
        'name': None,     # string - Site name
        'allowed_2fa_method_sms': None,     # boolean - Is SMS two factor authentication allowed?
        'allowed_2fa_method_totp': None,     # boolean - Is TOTP two factor authentication allowed?
        'allowed_2fa_method_u2f': None,     # boolean - Is U2F two factor authentication allowed?
        'allowed_2fa_method_yubi': None,     # boolean - Is yubikey two factor authentication allowed?
        'admin_user_id': None,     # int64 - User ID for the main site administrator
        'allow_bundle_names': None,     # boolean - Are manual Bundle names allowed?
        'allowed_countries': None,     # string - Comma seperated list of allowed Country codes
        'allowed_ips': None,     # string - List of allowed IP addresses
        'ask_about_overwrites': None,     # boolean - If false, rename conflicting files instead of asking for overwrite confirmation.  Only applies to web interface.
        'bundle_expiration': None,     # int64 - Site-wide Bundle expiration in days
        'bundle_password_required': None,     # boolean - Do Bundles require password protection?
        'color2_left': None,     # string - Page link and button color
        'color2_link': None,     # string - Top bar link color
        'color2_text': None,     # string - Page link and button color
        'color2_top': None,     # string - Top bar background color
        'color2_top_text': None,     # string - Top bar text color
        'created_at': None,     # date-time - Time this site was created
        'currency': None,     # string - Preferred currency
        'custom_namespace': None,     # boolean - Is this site using a custom namespace for users?
        'days_to_retain_backups': None,     # int64 - Number of days to keep deleted files
        'default_time_zone': None,     # string - Site default time zone
        'desktop_app': None,     # boolean - Is the desktop app enabled?
        'desktop_app_session_ip_pinning': None,     # boolean - Is desktop app session IP pinning enabled?
        'desktop_app_session_lifetime': None,     # int64 - Desktop app session lifetime (in hours)
        'disallowed_countries': None,     # string - Comma seperated list of disallowed Country codes
        'disable_notifications': None,     # boolean - Are notifications disabled?
        'disable_password_reset': None,     # boolean - Is password reset disabled?
        'domain': None,     # string - Custom domain
        'email': None,     # email - Main email for this site
        'reply_to_email': None,     # email - Reply-to email for this site
        'non_sso_groups_allowed': None,     # boolean - If true, groups can be manually created / modified / deleted by Site Admins. Otherwise, groups can only be managed via your SSO provider.
        'non_sso_users_allowed': None,     # boolean - If true, users can be manually created / modified / deleted by Site Admins. Otherwise, users can only be managed via your SSO provider.
        'folder_permissions_groups_only': None,     # boolean - If true, permissions for this site must be bound to a group (not a user). Otherwise, permissions must be bound to a user.
        'hipaa': None,     # boolean - Is there a signed HIPAA BAA between Files.com and this site?
        'icon128': None,     # Branded icon 128x128
        'icon16': None,     # Branded icon 16x16
        'icon32': None,     # Branded icon 32x32
        'icon48': None,     # Branded icon 48x48
        'immutable_files_set_at': None,     # date-time - Can files be modified?
        'include_password_in_welcome_email': None,     # boolean - Include password in emails to new users?
        'language': None,     # string - Site default language
        'ldap_base_dn': None,     # string - Base DN for looking up users in LDAP server
        'ldap_domain': None,     # string - Domain name that will be appended to usernames
        'ldap_enabled': None,     # boolean - Main LDAP setting: is LDAP enabled?
        'ldap_group_action': None,     # string - Should we sync groups from LDAP server?
        'ldap_group_exclusion': None,     # string - Comma or newline separated list of group names (with optional wildcards) to exclude when syncing.
        'ldap_group_inclusion': None,     # string - Comma or newline separated list of group names (with optional wildcards) to include when syncing.
        'ldap_host': None,     # string - LDAP host
        'ldap_host_2': None,     # string - LDAP backup host
        'ldap_host_3': None,     # string - LDAP backup host
        'ldap_port': None,     # int64 - LDAP port
        'ldap_secure': None,     # boolean - Use secure LDAP?
        'ldap_type': None,     # string - LDAP type
        'ldap_user_action': None,     # string - Should we sync users from LDAP server?
        'ldap_user_include_groups': None,     # string - Comma or newline separated list of group names (with optional wildcards) - if provided, only users in these groups will be added or synced.
        'ldap_username': None,     # string - Username for signing in to LDAP server.
        'ldap_username_field': None,     # string - LDAP username field
        'login_help_text': None,     # string - Login help text
        'logo': None,     # Branded logo
        'max_prior_passwords': None,     # int64 - Number of prior passwords to disallow
        'next_billing_amount': None,     # double - Next billing amount
        'next_billing_date': None,     # string - Next billing date
        'office_integration_available': None,     # boolean - Allow users to use Office for the web?
        'opt_out_global': None,     # boolean - Use servers in the USA only?
        'overage_notified_at': None,     # date-time - Last time the site was notified about an overage
        'overage_notify': None,     # boolean - Notify site email of overages?
        'overdue': None,     # boolean - Is this site's billing overdue?
        'password_min_length': None,     # int64 - Shortest password length for users
        'password_require_letter': None,     # boolean - Require a letter in passwords?
        'password_require_mixed': None,     # boolean - Require lower and upper case letters in passwords?
        'password_require_number': None,     # boolean - Require a number in passwords?
        'password_require_special': None,     # boolean - Require special characters in password?
        'password_require_unbreached': None,     # boolean - Require passwords that have not been previously breached? (see https://haveibeenpwned.com/)
        'password_requirements_apply_to_bundles': None,     # boolean - Require bundles' passwords, and passwords for other items (inboxes, public shares, etc.) to conform to the same requirements as users' passwords?
        'password_validity_days': None,     # int64 - Number of days password is valid
        'phone': None,     # string - Site phone number
        'require_2fa': None,     # boolean - Require two-factor authentication for all users?
        'require_2fa_stop_time': None,     # date-time - If set, requirement for two-factor authentication has been scheduled to end on this date-time.
        'require_2fa_user_type': None,     # string - What type of user is required to use two-factor authentication (when require_2fa is set to `true` for this site)?
        'session': None,     # Current session
        'session_pinned_by_ip': None,     # boolean - Are sessions locked to the same IP? (i.e. do users need to log in again if they change IPs?)
        'sftp_user_root_enabled': None,     # boolean - Use user FTP roots also for SFTP?
        'sharing_enabled': None,     # boolean - Allow bundle creation
        'show_request_access_link': None,     # boolean - Show request access link for users without access?  Currently unused.
        'site_footer': None,     # string - Custom site footer text
        'site_header': None,     # string - Custom site header text
        'smtp_address': None,     # string - SMTP server hostname or IP
        'smtp_authentication': None,     # string - SMTP server authentication type
        'smtp_from': None,     # string - From address to use when mailing through custom SMTP
        'smtp_port': None,     # int64 - SMTP server port
        'smtp_username': None,     # string - SMTP server username
        'session_expiry': None,     # double - Session expiry in hours
        'ssl_required': None,     # boolean - Is SSL required?  Disabling this is insecure.
        'subdomain': None,     # string - Site subdomain
        'switch_to_plan_date': None,     # date-time - If switching plans, when does the new plan take effect?
        'tls_disabled': None,     # boolean - Is TLS disabled(site setting)?
        'trial_days_left': None,     # int64 - Number of days left in trial
        'trial_until': None,     # date-time - When does this Site trial expire?
        'updated_at': None,     # date-time - Last time this Site was updated
        'use_provided_modified_at': None,     # boolean - Allow uploaders to set `provided_modified_at` for uploaded files?
        'user': None,     # User of current session
        'user_lockout': None,     # boolean - Will users be locked out after incorrect login attempts?
        'user_lockout_lock_period': None,     # int64 - How many hours to lock user out for failed password?
        'user_lockout_tries': None,     # int64 - Number of login tries within `user_lockout_within` hours before users are locked out
        'user_lockout_within': None,     # int64 - Number of hours for user lockout window
        'user_requests_enabled': None,     # boolean - Enable User Requests feature
        'welcome_custom_text': None,     # string - Custom text send in user welcome email
        'welcome_email_cc': None,     # email - Include this email in welcome emails if enabled
        'welcome_email_enabled': None,     # boolean - Will the welcome email be sent to new users?
        'welcome_screen': None,     # string - Does the welcome screen appear?
        'windows_mode_ftp': None,     # boolean - Does FTP user Windows emulation mode?
        'disable_users_from_inactivity_period_days': None,     # int64 - If greater than zero, users will unable to login if they do not show activity within this number of days.
    }

    def __init__(self, attributes=None, options=None):
        if not isinstance(attributes, dict):
            attributes = {}
        if not isinstance(options, dict):
            options = {}
        self.set_attributes(attributes)
        self.options = options

    def set_attributes(self, attributes):
        for (attribute, default_value) in Site.default_attributes.items():
            setattr(self, attribute, attributes.get(attribute, default_value))

    def get_attributes(self):
        return {k: getattr(self, k, None) for k in Site.default_attributes if getattr(self, k, None) is not None}


def get(params = None, options = None):
    if not isinstance(params, dict):
        params = {}
    if not isinstance(options, dict):
        options = {}
    response, options = Api.send_request("GET", "/site", params, options)
    return Site(response.data, options)

def get_usage(params = None, options = None):
    if not isinstance(params, dict):
        params = {}
    if not isinstance(options, dict):
        options = {}
    response, options = Api.send_request("GET", "/site/usage", params, options)
    return UsageSnapshot(response.data, options)

# Parameters:
#   name - string - Site name
#   subdomain - string - Site subdomain
#   domain - string - Custom domain
#   email - string - Main email for this site
#   reply_to_email - string - Reply-to email for this site
#   allow_bundle_names - boolean - Are manual Bundle names allowed?
#   bundle_expiration - int64 - Site-wide Bundle expiration in days
#   overage_notify - boolean - Notify site email of overages?
#   welcome_email_enabled - boolean - Will the welcome email be sent to new users?
#   ask_about_overwrites - boolean - If false, rename conflicting files instead of asking for overwrite confirmation.  Only applies to web interface.
#   show_request_access_link - boolean - Show request access link for users without access?  Currently unused.
#   welcome_email_cc - string - Include this email in welcome emails if enabled
#   welcome_custom_text - string - Custom text send in user welcome email
#   language - string - Site default language
#   windows_mode_ftp - boolean - Does FTP user Windows emulation mode?
#   default_time_zone - string - Site default time zone
#   desktop_app - boolean - Is the desktop app enabled?
#   desktop_app_session_ip_pinning - boolean - Is desktop app session IP pinning enabled?
#   desktop_app_session_lifetime - int64 - Desktop app session lifetime (in hours)
#   folder_permissions_groups_only - boolean - If true, permissions for this site must be bound to a group (not a user). Otherwise, permissions must be bound to a user.
#   welcome_screen - string - Does the welcome screen appear?
#   office_integration_available - boolean - Allow users to use Office for the web?
#   session_expiry - double - Session expiry in hours
#   ssl_required - boolean - Is SSL required?  Disabling this is insecure.
#   tls_disabled - boolean - Is TLS disabled(site setting)?
#   user_lockout - boolean - Will users be locked out after incorrect login attempts?
#   user_lockout_tries - int64 - Number of login tries within `user_lockout_within` hours before users are locked out
#   user_lockout_within - int64 - Number of hours for user lockout window
#   user_lockout_lock_period - int64 - How many hours to lock user out for failed password?
#   include_password_in_welcome_email - boolean - Include password in emails to new users?
#   allowed_countries - string - Comma seperated list of allowed Country codes
#   allowed_ips - string - List of allowed IP addresses
#   disallowed_countries - string - Comma seperated list of disallowed Country codes
#   days_to_retain_backups - int64 - Number of days to keep deleted files
#   max_prior_passwords - int64 - Number of prior passwords to disallow
#   password_validity_days - int64 - Number of days password is valid
#   password_min_length - int64 - Shortest password length for users
#   password_require_letter - boolean - Require a letter in passwords?
#   password_require_mixed - boolean - Require lower and upper case letters in passwords?
#   password_require_special - boolean - Require special characters in password?
#   password_require_number - boolean - Require a number in passwords?
#   password_require_unbreached - boolean - Require passwords that have not been previously breached? (see https://haveibeenpwned.com/)
#   sftp_user_root_enabled - boolean - Use user FTP roots also for SFTP?
#   disable_password_reset - boolean - Is password reset disabled?
#   immutable_files - boolean - Are files protected from modification?
#   session_pinned_by_ip - boolean - Are sessions locked to the same IP? (i.e. do users need to log in again if they change IPs?)
#   bundle_password_required - boolean - Do Bundles require password protection?
#   password_requirements_apply_to_bundles - boolean - Require bundles' passwords, and passwords for other items (inboxes, public shares, etc.) to conform to the same requirements as users' passwords?
#   opt_out_global - boolean - Use servers in the USA only?
#   use_provided_modified_at - boolean - Allow uploaders to set `provided_modified_at` for uploaded files?
#   custom_namespace - boolean - Is this site using a custom namespace for users?
#   disable_users_from_inactivity_period_days - int64 - If greater than zero, users will unable to login if they do not show activity within this number of days.
#   non_sso_groups_allowed - boolean - If true, groups can be manually created / modified / deleted by Site Admins. Otherwise, groups can only be managed via your SSO provider.
#   non_sso_users_allowed - boolean - If true, users can be manually created / modified / deleted by Site Admins. Otherwise, users can only be managed via your SSO provider.
#   sharing_enabled - boolean - Allow bundle creation
#   user_requests_enabled - boolean - Enable User Requests feature
#   allowed_2fa_method_sms - boolean - Is SMS two factor authentication allowed?
#   allowed_2fa_method_u2f - boolean - Is U2F two factor authentication allowed?
#   allowed_2fa_method_totp - boolean - Is TOTP two factor authentication allowed?
#   allowed_2fa_method_yubi - boolean - Is yubikey two factor authentication allowed?
#   require_2fa - boolean - Require two-factor authentication for all users?
#   require_2fa_user_type - string - What type of user is required to use two-factor authentication (when require_2fa is set to `true` for this site)?
#   color2_top - string - Top bar background color
#   color2_left - string - Page link and button color
#   color2_link - string - Top bar link color
#   color2_text - string - Page link and button color
#   color2_top_text - string - Top bar text color
#   site_header - string - Custom site header text
#   site_footer - string - Custom site footer text
#   login_help_text - string - Login help text
#   smtp_address - string - SMTP server hostname or IP
#   smtp_authentication - string - SMTP server authentication type
#   smtp_from - string - From address to use when mailing through custom SMTP
#   smtp_username - string - SMTP server username
#   smtp_port - int64 - SMTP server port
#   ldap_enabled - boolean - Main LDAP setting: is LDAP enabled?
#   ldap_type - string - LDAP type
#   ldap_host - string - LDAP host
#   ldap_host_2 - string - LDAP backup host
#   ldap_host_3 - string - LDAP backup host
#   ldap_port - int64 - LDAP port
#   ldap_secure - boolean - Use secure LDAP?
#   ldap_username - string - Username for signing in to LDAP server.
#   ldap_username_field - string - LDAP username field
#   ldap_domain - string - Domain name that will be appended to usernames
#   ldap_user_action - string - Should we sync users from LDAP server?
#   ldap_group_action - string - Should we sync groups from LDAP server?
#   ldap_user_include_groups - string - Comma or newline separated list of group names (with optional wildcards) - if provided, only users in these groups will be added or synced.
#   ldap_group_exclusion - string - Comma or newline separated list of group names (with optional wildcards) to exclude when syncing.
#   ldap_group_inclusion - string - Comma or newline separated list of group names (with optional wildcards) to include when syncing.
#   ldap_base_dn - string - Base DN for looking up users in LDAP server
#   icon16_file - file
#   icon16_delete - boolean - If true, will delete the file stored in icon16
#   icon32_file - file
#   icon32_delete - boolean - If true, will delete the file stored in icon32
#   icon48_file - file
#   icon48_delete - boolean - If true, will delete the file stored in icon48
#   icon128_file - file
#   icon128_delete - boolean - If true, will delete the file stored in icon128
#   logo_file - file
#   logo_delete - boolean - If true, will delete the file stored in logo
#   disable_2fa_with_delay - boolean - If set to true, we will begin the process of disabling 2FA on this site.
#   ldap_password_change - string - New LDAP password.
#   ldap_password_change_confirmation - string - Confirm new LDAP password.
#   smtp_password - string - Password for SMTP server.
def update(params = None, options = None):
    if not isinstance(params, dict):
        params = {}
    if not isinstance(options, dict):
        options = {}
    if "name" in params and not isinstance(params["name"], str):
        raise InvalidParameterError("Bad parameter: name must be an str")
    if "subdomain" in params and not isinstance(params["subdomain"], str):
        raise InvalidParameterError("Bad parameter: subdomain must be an str")
    if "domain" in params and not isinstance(params["domain"], str):
        raise InvalidParameterError("Bad parameter: domain must be an str")
    if "email" in params and not isinstance(params["email"], str):
        raise InvalidParameterError("Bad parameter: email must be an str")
    if "reply_to_email" in params and not isinstance(params["reply_to_email"], str):
        raise InvalidParameterError("Bad parameter: reply_to_email must be an str")
    if "bundle_expiration" in params and not isinstance(params["bundle_expiration"], int):
        raise InvalidParameterError("Bad parameter: bundle_expiration must be an int")
    if "welcome_email_cc" in params and not isinstance(params["welcome_email_cc"], str):
        raise InvalidParameterError("Bad parameter: welcome_email_cc must be an str")
    if "welcome_custom_text" in params and not isinstance(params["welcome_custom_text"], str):
        raise InvalidParameterError("Bad parameter: welcome_custom_text must be an str")
    if "language" in params and not isinstance(params["language"], str):
        raise InvalidParameterError("Bad parameter: language must be an str")
    if "default_time_zone" in params and not isinstance(params["default_time_zone"], str):
        raise InvalidParameterError("Bad parameter: default_time_zone must be an str")
    if "desktop_app_session_lifetime" in params and not isinstance(params["desktop_app_session_lifetime"], int):
        raise InvalidParameterError("Bad parameter: desktop_app_session_lifetime must be an int")
    if "welcome_screen" in params and not isinstance(params["welcome_screen"], str):
        raise InvalidParameterError("Bad parameter: welcome_screen must be an str")
    if "session_expiry" in params and not isinstance(params["session_expiry"], float):
        raise InvalidParameterError("Bad parameter: session_expiry must be an float")
    if "user_lockout_tries" in params and not isinstance(params["user_lockout_tries"], int):
        raise InvalidParameterError("Bad parameter: user_lockout_tries must be an int")
    if "user_lockout_within" in params and not isinstance(params["user_lockout_within"], int):
        raise InvalidParameterError("Bad parameter: user_lockout_within must be an int")
    if "user_lockout_lock_period" in params and not isinstance(params["user_lockout_lock_period"], int):
        raise InvalidParameterError("Bad parameter: user_lockout_lock_period must be an int")
    if "allowed_countries" in params and not isinstance(params["allowed_countries"], str):
        raise InvalidParameterError("Bad parameter: allowed_countries must be an str")
    if "allowed_ips" in params and not isinstance(params["allowed_ips"], str):
        raise InvalidParameterError("Bad parameter: allowed_ips must be an str")
    if "disallowed_countries" in params and not isinstance(params["disallowed_countries"], str):
        raise InvalidParameterError("Bad parameter: disallowed_countries must be an str")
    if "days_to_retain_backups" in params and not isinstance(params["days_to_retain_backups"], int):
        raise InvalidParameterError("Bad parameter: days_to_retain_backups must be an int")
    if "max_prior_passwords" in params and not isinstance(params["max_prior_passwords"], int):
        raise InvalidParameterError("Bad parameter: max_prior_passwords must be an int")
    if "password_validity_days" in params and not isinstance(params["password_validity_days"], int):
        raise InvalidParameterError("Bad parameter: password_validity_days must be an int")
    if "password_min_length" in params and not isinstance(params["password_min_length"], int):
        raise InvalidParameterError("Bad parameter: password_min_length must be an int")
    if "disable_users_from_inactivity_period_days" in params and not isinstance(params["disable_users_from_inactivity_period_days"], int):
        raise InvalidParameterError("Bad parameter: disable_users_from_inactivity_period_days must be an int")
    if "require_2fa_user_type" in params and not isinstance(params["require_2fa_user_type"], str):
        raise InvalidParameterError("Bad parameter: require_2fa_user_type must be an str")
    if "color2_top" in params and not isinstance(params["color2_top"], str):
        raise InvalidParameterError("Bad parameter: color2_top must be an str")
    if "color2_left" in params and not isinstance(params["color2_left"], str):
        raise InvalidParameterError("Bad parameter: color2_left must be an str")
    if "color2_link" in params and not isinstance(params["color2_link"], str):
        raise InvalidParameterError("Bad parameter: color2_link must be an str")
    if "color2_text" in params and not isinstance(params["color2_text"], str):
        raise InvalidParameterError("Bad parameter: color2_text must be an str")
    if "color2_top_text" in params and not isinstance(params["color2_top_text"], str):
        raise InvalidParameterError("Bad parameter: color2_top_text must be an str")
    if "site_header" in params and not isinstance(params["site_header"], str):
        raise InvalidParameterError("Bad parameter: site_header must be an str")
    if "site_footer" in params and not isinstance(params["site_footer"], str):
        raise InvalidParameterError("Bad parameter: site_footer must be an str")
    if "login_help_text" in params and not isinstance(params["login_help_text"], str):
        raise InvalidParameterError("Bad parameter: login_help_text must be an str")
    if "smtp_address" in params and not isinstance(params["smtp_address"], str):
        raise InvalidParameterError("Bad parameter: smtp_address must be an str")
    if "smtp_authentication" in params and not isinstance(params["smtp_authentication"], str):
        raise InvalidParameterError("Bad parameter: smtp_authentication must be an str")
    if "smtp_from" in params and not isinstance(params["smtp_from"], str):
        raise InvalidParameterError("Bad parameter: smtp_from must be an str")
    if "smtp_username" in params and not isinstance(params["smtp_username"], str):
        raise InvalidParameterError("Bad parameter: smtp_username must be an str")
    if "smtp_port" in params and not isinstance(params["smtp_port"], int):
        raise InvalidParameterError("Bad parameter: smtp_port must be an int")
    if "ldap_type" in params and not isinstance(params["ldap_type"], str):
        raise InvalidParameterError("Bad parameter: ldap_type must be an str")
    if "ldap_host" in params and not isinstance(params["ldap_host"], str):
        raise InvalidParameterError("Bad parameter: ldap_host must be an str")
    if "ldap_host_2" in params and not isinstance(params["ldap_host_2"], str):
        raise InvalidParameterError("Bad parameter: ldap_host_2 must be an str")
    if "ldap_host_3" in params and not isinstance(params["ldap_host_3"], str):
        raise InvalidParameterError("Bad parameter: ldap_host_3 must be an str")
    if "ldap_port" in params and not isinstance(params["ldap_port"], int):
        raise InvalidParameterError("Bad parameter: ldap_port must be an int")
    if "ldap_username" in params and not isinstance(params["ldap_username"], str):
        raise InvalidParameterError("Bad parameter: ldap_username must be an str")
    if "ldap_username_field" in params and not isinstance(params["ldap_username_field"], str):
        raise InvalidParameterError("Bad parameter: ldap_username_field must be an str")
    if "ldap_domain" in params and not isinstance(params["ldap_domain"], str):
        raise InvalidParameterError("Bad parameter: ldap_domain must be an str")
    if "ldap_user_action" in params and not isinstance(params["ldap_user_action"], str):
        raise InvalidParameterError("Bad parameter: ldap_user_action must be an str")
    if "ldap_group_action" in params and not isinstance(params["ldap_group_action"], str):
        raise InvalidParameterError("Bad parameter: ldap_group_action must be an str")
    if "ldap_user_include_groups" in params and not isinstance(params["ldap_user_include_groups"], str):
        raise InvalidParameterError("Bad parameter: ldap_user_include_groups must be an str")
    if "ldap_group_exclusion" in params and not isinstance(params["ldap_group_exclusion"], str):
        raise InvalidParameterError("Bad parameter: ldap_group_exclusion must be an str")
    if "ldap_group_inclusion" in params and not isinstance(params["ldap_group_inclusion"], str):
        raise InvalidParameterError("Bad parameter: ldap_group_inclusion must be an str")
    if "ldap_base_dn" in params and not isinstance(params["ldap_base_dn"], str):
        raise InvalidParameterError("Bad parameter: ldap_base_dn must be an str")
    if "ldap_password_change" in params and not isinstance(params["ldap_password_change"], str):
        raise InvalidParameterError("Bad parameter: ldap_password_change must be an str")
    if "ldap_password_change_confirmation" in params and not isinstance(params["ldap_password_change_confirmation"], str):
        raise InvalidParameterError("Bad parameter: ldap_password_change_confirmation must be an str")
    if "smtp_password" in params and not isinstance(params["smtp_password"], str):
        raise InvalidParameterError("Bad parameter: smtp_password must be an str")
    response, options = Api.send_request("PATCH", "/site", params, options)
    return Site(response.data, options)

def new(*args, **kwargs):
    return Site(*args, **kwargs)


import datetime
from files_sdk.api import Api
from files_sdk.list_obj import ListObj
from files_sdk.exceptions import InvalidParameterError, MissingParameterError, NotImplementedError

class HistoryExportResult:
    default_attributes = {
        'id': None,     # int64 - Action ID
        'created_at': None,     # int64 - When the action happened
        'user_id': None,     # int64 - User ID
        'file_id': None,     # int64 - File ID related to the action
        'parent_id': None,     # int64 - ID of the parent folder
        'path': None,     # string - Path of the related action This must be slash-delimited, but it must neither start nor end with a slash. Maximum of 5000 characters.
        'folder': None,     # string - Folder in which the action occurred
        'src': None,     # string - File move originated from this path
        'destination': None,     # string - File moved to this destination folder
        'ip': None,     # string - Client IP that performed the action
        'username': None,     # string - Username of the user that performed the action
        'action': None,     # string - What action was taken. Valid values: `create`, `read`, `update`, `destroy`, `move`, `login`, `failedlogin`, `copy`, `user_create`, `user_update`, `user_destroy`, `group_create`, `group_update`, `group_destroy`, `permission_create`, `permission_destroy`, `api_key_create`, `api_key_update`, `api_key_destroy`
        'failure_type': None,     # string - The type of login failure, if applicable.  Valid values: `expired_trial`, `account_overdue`, `locked_out`, `ip_mismatch`, `password_mismatch`, `site_mismatch`, `username_not_found`, `none`, `no_ftp_permission`, `no_web_permission`, `no_directory`, `errno_enoent`, `no_sftp_permission`, `no_dav_permission`, `no_restapi_permission`, `key_mismatch`, `region_mismatch`, `expired_access`, `desktop_ip_mismatch`, `desktop_api_key_not_used_quickly_enough`, `disabled`, `country_mismatch`
        'interface': None,     # string - Inteface through which the action was taken. Valid values: `web`, `ftp`, `robot`, `jsapi`, `webdesktopapi`, `sftp`, `dav`, `desktop`, `restapi`, `scim`, `office`
        'target_id': None,     # int64 - ID of the object (such as Users, or API Keys) on which the action was taken
        'target_name': None,     # string - Name of the User, Group or other object with a name related to this action
        'target_permission': None,     # string - Permission level of the action
        'target_recursive': None,     # boolean - Whether or not the action was recursive
        'target_expires_at': None,     # int64 - If searching for Histories about API keys, this is when the API key will expire
        'target_permission_set': None,     # string - If searching for Histories about API keys, this represents the permission set of the associated  API key
        'target_platform': None,     # string - If searching for Histories about API keys, this is the platform on which the action was taken
        'target_username': None,     # string - If searching for Histories about API keys, this is the username on which the action was taken
        'target_user_id': None,     # int64 - If searching for Histories about API keys, this is the User ID on which the action was taken
    }

    def __init__(self, attributes=None, options=None):
        if not isinstance(attributes, dict):
            attributes = {}
        if not isinstance(options, dict):
            options = {}
        self.set_attributes(attributes)
        self.options = options

    def set_attributes(self, attributes):
        for (attribute, default_value) in HistoryExportResult.default_attributes.items():
            setattr(self, attribute, attributes.get(attribute, default_value))

    def get_attributes(self):
        return {k: getattr(self, k, None) for k in HistoryExportResult.default_attributes if getattr(self, k, None) is not None}


# Parameters:
#   user_id - int64 - User ID.  Provide a value of `0` to operate the current session's user.
#   cursor - string - Used for pagination.  Send a cursor value to resume an existing list from the point at which you left off.  Get a cursor from an existing list via the X-Files-Cursor-Next header.
#   per_page - int64 - Number of records to show per page.  (Max: 10,000, 1,000 or less is recommended).
#   history_export_id (required) - int64 - ID of the associated history export.
def list(params = None, options = None):
    if not isinstance(params, dict):
        params = {}
    if not isinstance(options, dict):
        options = {}
    if "user_id" in params and not isinstance(params["user_id"], int):
        raise InvalidParameterError("Bad parameter: user_id must be an int")
    if "cursor" in params and not isinstance(params["cursor"], str):
        raise InvalidParameterError("Bad parameter: cursor must be an str")
    if "per_page" in params and not isinstance(params["per_page"], int):
        raise InvalidParameterError("Bad parameter: per_page must be an int")
    if "history_export_id" in params and not isinstance(params["history_export_id"], int):
        raise InvalidParameterError("Bad parameter: history_export_id must be an int")
    if "history_export_id" not in params:
        raise MissingParameterError("Parameter missing: history_export_id")
    return ListObj(HistoryExportResult,"GET", "/history_export_results", params, options)

def all(params = None, options = None):
    list(params, options)

def new(*args, **kwargs):
    return HistoryExportResult(*args, **kwargs)


import datetime
from files_sdk.api import Api
from files_sdk.list_obj import ListObj
from files_sdk.exceptions import InvalidParameterError, MissingParameterError, NotImplementedError

class InboxRegistration:
    default_attributes = {
        'code': None,     # string - Registration cookie code
        'name': None,     # string - Registrant name
        'company': None,     # string - Registrant company name
        'email': None,     # string - Registrant email address
        'form_field_set_id': None,     # int64 - Id of associated form field set
        'form_field_data': None,     # string - Data for form field set with form field ids as keys and user data as values
    }

    def __init__(self, attributes=None, options=None):
        if not isinstance(attributes, dict):
            attributes = {}
        if not isinstance(options, dict):
            options = {}
        self.set_attributes(attributes)
        self.options = options

    def set_attributes(self, attributes):
        for (attribute, default_value) in InboxRegistration.default_attributes.items():
            setattr(self, attribute, attributes.get(attribute, default_value))

    def get_attributes(self):
        return {k: getattr(self, k, None) for k in InboxRegistration.default_attributes if getattr(self, k, None) is not None}


# Parameters:
#   cursor - string - Used for pagination.  Send a cursor value to resume an existing list from the point at which you left off.  Get a cursor from an existing list via the X-Files-Cursor-Next header.
#   per_page - int64 - Number of records to show per page.  (Max: 10,000, 1,000 or less is recommended).
#   folder_behavior_id (required) - int64 - ID of the associated Inbox.
def list(params = None, options = None):
    if not isinstance(params, dict):
        params = {}
    if not isinstance(options, dict):
        options = {}
    if "cursor" in params and not isinstance(params["cursor"], str):
        raise InvalidParameterError("Bad parameter: cursor must be an str")
    if "per_page" in params and not isinstance(params["per_page"], int):
        raise InvalidParameterError("Bad parameter: per_page must be an int")
    if "folder_behavior_id" in params and not isinstance(params["folder_behavior_id"], int):
        raise InvalidParameterError("Bad parameter: folder_behavior_id must be an int")
    if "folder_behavior_id" not in params:
        raise MissingParameterError("Parameter missing: folder_behavior_id")
    return ListObj(InboxRegistration,"GET", "/inbox_registrations", params, options)

def all(params = None, options = None):
    list(params, options)

def new(*args, **kwargs):
    return InboxRegistration(*args, **kwargs)


import datetime
from files_sdk.api import Api
from files_sdk.list_obj import ListObj
from files_sdk.exceptions import InvalidParameterError, MissingParameterError, NotImplementedError

class ApiKey:
    default_attributes = {
        'id': None,     # int64 - API Key ID
        'descriptive_label': None,     # string - Unique label that describes this API key.  Useful for external systems where you may have API keys from multiple accounts and want a human-readable label for each key.
        'created_at': None,     # date-time - Time which API Key was created
        'expires_at': None,     # date-time - API Key expiration date
        'key': None,     # string - API Key actual key string
        'last_use_at': None,     # date-time - API Key last used - note this value is only updated once per 3 hour period, so the 'actual' time of last use may be up to 3 hours later than this timestamp.
        'name': None,     # string - Internal name for the API Key.  For your use.
        'path': None,     # string - Folder path restriction for this api key. This must be slash-delimited, but it must neither start nor end with a slash. Maximum of 5000 characters.
        'permission_set': None,     # string - Permissions for this API Key.  Keys with the `desktop_app` permission set only have the ability to do the functions provided in our Desktop App (File and Share Link operations).  Additional permission sets may become available in the future, such as for a Site Admin to give a key with no administrator privileges.  If you have ideas for permission sets, please let us know.
        'platform': None,     # string - If this API key represents a Desktop app, what platform was it created on?
        'user_id': None,     # int64 - User ID for the owner of this API Key.  May be blank for Site-wide API Keys.
    }

    def __init__(self, attributes=None, options=None):
        if not isinstance(attributes, dict):
            attributes = {}
        if not isinstance(options, dict):
            options = {}
        self.set_attributes(attributes)
        self.options = options

    def set_attributes(self, attributes):
        for (attribute, default_value) in ApiKey.default_attributes.items():
            setattr(self, attribute, attributes.get(attribute, default_value))

    def get_attributes(self):
        return {k: getattr(self, k, None) for k in ApiKey.default_attributes if getattr(self, k, None) is not None}

    # Parameters:
    #   name - string - Internal name for the API Key.  For your use.
    #   expires_at - string - API Key expiration date
    #   permission_set - string - Permissions for this API Key.  Keys with the `desktop_app` permission set only have the ability to do the functions provided in our Desktop App (File and Share Link operations).  Additional permission sets may become available in the future, such as for a Site Admin to give a key with no administrator privileges.  If you have ideas for permission sets, please let us know.
    def update(self, params = None):
        if not isinstance(params, dict):
            params = {}

        if hasattr(self, "id") and self.id:
            params['id'] = self.id
        else:
            raise MissingParameterError("Current object doesn't have a id")
        if "id" not in params:
            raise MissingParameterError("Parameter missing: id")
        if "id" in params and not isinstance(params["id"], int):
            raise InvalidParameterError("Bad parameter: id must be an int")
        if "name" in params and not isinstance(params["name"], str):
            raise InvalidParameterError("Bad parameter: name must be an str")
        if "expires_at" in params and not isinstance(params["expires_at"], str):
            raise InvalidParameterError("Bad parameter: expires_at must be an str")
        if "permission_set" in params and not isinstance(params["permission_set"], str):
            raise InvalidParameterError("Bad parameter: permission_set must be an str")
        response, _options = Api.send_request("PATCH", "/api_keys/{id}".format(id=params['id']), params, self.options)
        return response.data

    def delete(self, params = None):
        if not isinstance(params, dict):
            params = {}

        if hasattr(self, "id") and self.id:
            params['id'] = self.id
        else:
            raise MissingParameterError("Current object doesn't have a id")
        if "id" not in params:
            raise MissingParameterError("Parameter missing: id")
        if "id" in params and not isinstance(params["id"], int):
            raise InvalidParameterError("Bad parameter: id must be an int")
        response, _options = Api.send_request("DELETE", "/api_keys/{id}".format(id=params['id']), params, self.options)
        return response.data

    def destroy(self, params = None):
        self.delete(params)

    def save(self):
        if hasattr(self, "id") and self.id:
            self.update(self.get_attributes())
        else:
            new_obj = create(self.get_attributes(), self.options)
            self.set_attributes(new_obj.get_attributes())

# Parameters:
#   user_id - int64 - User ID.  Provide a value of `0` to operate the current session's user.
#   cursor - string - Used for pagination.  Send a cursor value to resume an existing list from the point at which you left off.  Get a cursor from an existing list via the X-Files-Cursor-Next header.
#   per_page - int64 - Number of records to show per page.  (Max: 10,000, 1,000 or less is recommended).
#   sort_by - object - If set, sort records by the specified field in either 'asc' or 'desc' direction (e.g. sort_by[last_login_at]=desc). Valid fields are `expires_at`.
#   filter - object - If set, return records where the specifiied field is equal to the supplied value. Valid fields are `expires_at`.
#   filter_gt - object - If set, return records where the specifiied field is greater than the supplied value. Valid fields are `expires_at`.
#   filter_gteq - object - If set, return records where the specifiied field is greater than or equal to the supplied value. Valid fields are `expires_at`.
#   filter_like - object - If set, return records where the specifiied field is equal to the supplied value. Valid fields are `expires_at`.
#   filter_lt - object - If set, return records where the specifiied field is less than the supplied value. Valid fields are `expires_at`.
#   filter_lteq - object - If set, return records where the specifiied field is less than or equal to the supplied value. Valid fields are `expires_at`.
def list(params = None, options = None):
    if not isinstance(params, dict):
        params = {}
    if not isinstance(options, dict):
        options = {}
    if "user_id" in params and not isinstance(params["user_id"], int):
        raise InvalidParameterError("Bad parameter: user_id must be an int")
    if "cursor" in params and not isinstance(params["cursor"], str):
        raise InvalidParameterError("Bad parameter: cursor must be an str")
    if "per_page" in params and not isinstance(params["per_page"], int):
        raise InvalidParameterError("Bad parameter: per_page must be an int")
    if "sort_by" in params and not isinstance(params["sort_by"], dict):
        raise InvalidParameterError("Bad parameter: sort_by must be an dict")
    if "filter" in params and not isinstance(params["filter"], dict):
        raise InvalidParameterError("Bad parameter: filter must be an dict")
    if "filter_gt" in params and not isinstance(params["filter_gt"], dict):
        raise InvalidParameterError("Bad parameter: filter_gt must be an dict")
    if "filter_gteq" in params and not isinstance(params["filter_gteq"], dict):
        raise InvalidParameterError("Bad parameter: filter_gteq must be an dict")
    if "filter_like" in params and not isinstance(params["filter_like"], dict):
        raise InvalidParameterError("Bad parameter: filter_like must be an dict")
    if "filter_lt" in params and not isinstance(params["filter_lt"], dict):
        raise InvalidParameterError("Bad parameter: filter_lt must be an dict")
    if "filter_lteq" in params and not isinstance(params["filter_lteq"], dict):
        raise InvalidParameterError("Bad parameter: filter_lteq must be an dict")
    return ListObj(ApiKey,"GET", "/api_keys", params, options)

def all(params = None, options = None):
    list(params, options)

def find_current(params = None, options = None):
    if not isinstance(params, dict):
        params = {}
    if not isinstance(options, dict):
        options = {}
    response, options = Api.send_request("GET", "/api_key", params, options)
    return ApiKey(response.data, options)

# Parameters:
#   id (required) - int64 - Api Key ID.
def find(id, params = None, options = None):
    if not isinstance(params, dict):
        params = {}
    if not isinstance(options, dict):
        options = {}
    params["id"] = id
    if "id" in params and not isinstance(params["id"], int):
        raise InvalidParameterError("Bad parameter: id must be an int")
    if "id" not in params:
        raise MissingParameterError("Parameter missing: id")
    response, options = Api.send_request("GET", "/api_keys/{id}".format(id=params['id']), params, options)
    return ApiKey(response.data, options)

def get(id, params = None, options = None):
    find(id, params, options)

# Parameters:
#   user_id - int64 - User ID.  Provide a value of `0` to operate the current session's user.
#   name - string - Internal name for the API Key.  For your use.
#   expires_at - string - API Key expiration date
#   permission_set - string - Permissions for this API Key.  Keys with the `desktop_app` permission set only have the ability to do the functions provided in our Desktop App (File and Share Link operations).  Additional permission sets may become available in the future, such as for a Site Admin to give a key with no administrator privileges.  If you have ideas for permission sets, please let us know.
#   path - string - Folder path restriction for this api key.
def create(params = None, options = None):
    if not isinstance(params, dict):
        params = {}
    if not isinstance(options, dict):
        options = {}
    if "user_id" in params and not isinstance(params["user_id"], int):
        raise InvalidParameterError("Bad parameter: user_id must be an int")
    if "name" in params and not isinstance(params["name"], str):
        raise InvalidParameterError("Bad parameter: name must be an str")
    if "expires_at" in params and not isinstance(params["expires_at"], str):
        raise InvalidParameterError("Bad parameter: expires_at must be an str")
    if "permission_set" in params and not isinstance(params["permission_set"], str):
        raise InvalidParameterError("Bad parameter: permission_set must be an str")
    if "path" in params and not isinstance(params["path"], str):
        raise InvalidParameterError("Bad parameter: path must be an str")
    response, options = Api.send_request("POST", "/api_keys", params, options)
    return ApiKey(response.data, options)

# Parameters:
#   expires_at - string - API Key expiration date
#   name - string - Internal name for the API Key.  For your use.
#   permission_set - string - Permissions for this API Key.  Keys with the `desktop_app` permission set only have the ability to do the functions provided in our Desktop App (File and Share Link operations).  Additional permission sets may become available in the future, such as for a Site Admin to give a key with no administrator privileges.  If you have ideas for permission sets, please let us know.
def update_current(params = None, options = None):
    if not isinstance(params, dict):
        params = {}
    if not isinstance(options, dict):
        options = {}
    if "expires_at" in params and not isinstance(params["expires_at"], str):
        raise InvalidParameterError("Bad parameter: expires_at must be an str")
    if "name" in params and not isinstance(params["name"], str):
        raise InvalidParameterError("Bad parameter: name must be an str")
    if "permission_set" in params and not isinstance(params["permission_set"], str):
        raise InvalidParameterError("Bad parameter: permission_set must be an str")
    response, options = Api.send_request("PATCH", "/api_key", params, options)
    return ApiKey(response.data, options)

# Parameters:
#   name - string - Internal name for the API Key.  For your use.
#   expires_at - string - API Key expiration date
#   permission_set - string - Permissions for this API Key.  Keys with the `desktop_app` permission set only have the ability to do the functions provided in our Desktop App (File and Share Link operations).  Additional permission sets may become available in the future, such as for a Site Admin to give a key with no administrator privileges.  If you have ideas for permission sets, please let us know.
def update(id, params = None, options = None):
    if not isinstance(params, dict):
        params = {}
    if not isinstance(options, dict):
        options = {}
    params["id"] = id
    if "id" in params and not isinstance(params["id"], int):
        raise InvalidParameterError("Bad parameter: id must be an int")
    if "name" in params and not isinstance(params["name"], str):
        raise InvalidParameterError("Bad parameter: name must be an str")
    if "expires_at" in params and not isinstance(params["expires_at"], str):
        raise InvalidParameterError("Bad parameter: expires_at must be an str")
    if "permission_set" in params and not isinstance(params["permission_set"], str):
        raise InvalidParameterError("Bad parameter: permission_set must be an str")
    if "id" not in params:
        raise MissingParameterError("Parameter missing: id")
    response, options = Api.send_request("PATCH", "/api_keys/{id}".format(id=params['id']), params, options)
    return ApiKey(response.data, options)

def delete_current(params = None, options = None):
    if not isinstance(params, dict):
        params = {}
    if not isinstance(options, dict):
        options = {}
    response, _options = Api.send_request("DELETE", "/api_key", params, options)
    return response.data

def delete(id, params = None, options = None):
    if not isinstance(params, dict):
        params = {}
    if not isinstance(options, dict):
        options = {}
    params["id"] = id
    if "id" in params and not isinstance(params["id"], int):
        raise InvalidParameterError("Bad parameter: id must be an int")
    if "id" not in params:
        raise MissingParameterError("Parameter missing: id")
    response, _options = Api.send_request("DELETE", "/api_keys/{id}".format(id=params['id']), params, options)
    return response.data

def destroy(id, params = None, options = None):
    delete(id, params, options)

def new(*args, **kwargs):
    return ApiKey(*args, **kwargs)


import datetime
from files_sdk.api import Api
from files_sdk.exceptions import InvalidParameterError, MissingParameterError, NotImplementedError

class FormField:
    default_attributes = {
        'id': None,     # int64 - Form field id
        'label': None,     # string - Label to be displayed
        'required': None,     # boolean - Is this a required field?
        'help_text': None,     # string - Help text to be displayed
        'field_type': None,     # string - Type of Field
        'options_for_select': None,     # string - Options to display for radio and dropdown
        'default_option': None,     # string - Default option for radio and dropdown
        'form_field_set_id': None,     # int64 - Form field set id
    }

    def __init__(self, attributes=None, options=None):
        if not isinstance(attributes, dict):
            attributes = {}
        if not isinstance(options, dict):
            options = {}
        self.set_attributes(attributes)
        self.options = options

    def set_attributes(self, attributes):
        for (attribute, default_value) in FormField.default_attributes.items():
            setattr(self, attribute, attributes.get(attribute, default_value))

    def get_attributes(self):
        return {k: getattr(self, k, None) for k in FormField.default_attributes if getattr(self, k, None) is not None}


def new(*args, **kwargs):
    return FormField(*args, **kwargs)


import datetime
from files_sdk.api import Api
from files_sdk.exceptions import InvalidParameterError, MissingParameterError, NotImplementedError

class Style:
    default_attributes = {
        'id': None,     # int64 - Style ID
        'path': None,     # string - Folder path This must be slash-delimited, but it must neither start nor end with a slash. Maximum of 5000 characters.
        'logo': None,     # Logo
        'thumbnail': None,     # Logo thumbnail
        'file': None,     # file - Logo for custom branding.
    }

    def __init__(self, attributes=None, options=None):
        if not isinstance(attributes, dict):
            attributes = {}
        if not isinstance(options, dict):
            options = {}
        self.set_attributes(attributes)
        self.options = options

    def set_attributes(self, attributes):
        for (attribute, default_value) in Style.default_attributes.items():
            setattr(self, attribute, attributes.get(attribute, default_value))

    def get_attributes(self):
        return {k: getattr(self, k, None) for k in Style.default_attributes if getattr(self, k, None) is not None}

    # Parameters:
    #   file (required) - file - Logo for custom branding.
    def update(self, params = None):
        if not isinstance(params, dict):
            params = {}

        if hasattr(self, "path") and self.path:
            params['path'] = self.path
        else:
            raise MissingParameterError("Current object doesn't have a path")
        if "path" not in params:
            raise MissingParameterError("Parameter missing: path")
        if "file" not in params:
            raise MissingParameterError("Parameter missing: file")
        if "path" in params and not isinstance(params["path"], str):
            raise InvalidParameterError("Bad parameter: path must be an str")
        response, _options = Api.send_request("PATCH", "/styles/{path}".format(path=params['path']), params, self.options)
        return response.data

    def delete(self, params = None):
        if not isinstance(params, dict):
            params = {}

        if hasattr(self, "path") and self.path:
            params['path'] = self.path
        else:
            raise MissingParameterError("Current object doesn't have a path")
        if "path" not in params:
            raise MissingParameterError("Parameter missing: path")
        if "path" in params and not isinstance(params["path"], str):
            raise InvalidParameterError("Bad parameter: path must be an str")
        response, _options = Api.send_request("DELETE", "/styles/{path}".format(path=params['path']), params, self.options)
        return response.data

    def destroy(self, params = None):
        self.delete(params)

    def save(self):
        self.update(self.get_attributes())

# Parameters:
#   path (required) - string - Style path.
def find(path, params = None, options = None):
    if not isinstance(params, dict):
        params = {}
    if not isinstance(options, dict):
        options = {}
    params["path"] = path
    if "path" in params and not isinstance(params["path"], str):
        raise InvalidParameterError("Bad parameter: path must be an str")
    if "path" not in params:
        raise MissingParameterError("Parameter missing: path")
    response, options = Api.send_request("GET", "/styles/{path}".format(path=params['path']), params, options)
    return Style(response.data, options)

def get(path, params = None, options = None):
    find(path, params, options)

# Parameters:
#   file (required) - file - Logo for custom branding.
def update(path, params = None, options = None):
    if not isinstance(params, dict):
        params = {}
    if not isinstance(options, dict):
        options = {}
    params["path"] = path
    if "path" in params and not isinstance(params["path"], str):
        raise InvalidParameterError("Bad parameter: path must be an str")
    if "path" not in params:
        raise MissingParameterError("Parameter missing: path")
    if "file" not in params:
        raise MissingParameterError("Parameter missing: file")
    response, options = Api.send_request("PATCH", "/styles/{path}".format(path=params['path']), params, options)
    return Style(response.data, options)

def delete(path, params = None, options = None):
    if not isinstance(params, dict):
        params = {}
    if not isinstance(options, dict):
        options = {}
    params["path"] = path
    if "path" in params and not isinstance(params["path"], str):
        raise InvalidParameterError("Bad parameter: path must be an str")
    if "path" not in params:
        raise MissingParameterError("Parameter missing: path")
    response, _options = Api.send_request("DELETE", "/styles/{path}".format(path=params['path']), params, options)
    return response.data

def destroy(path, params = None, options = None):
    delete(path, params, options)

def new(*args, **kwargs):
    return Style(*args, **kwargs)


from setuptools import setup, find_packages

with open('_VERSION') as version_file:
    version = version_file.read().strip()

setup(
    name='Files.com',
    version=version,
    license='MIT',
    description="Python bindings for the Files.com API",
    packages = find_packages(exclude = ('tests', 'tests.*', 'doc')),
    install_requires=[
        'requests >= 2.20; python_version >= "3.0"',
    ],
    python_requires="!=3.0.*, !=3.1.*, !=3.2.*, !=3.3.*, !=3.4.*",
)



import datetime
from files_sdk.api import Api
from files_sdk.list_obj import ListObj
from files_sdk.exceptions import InvalidParameterError, MissingParameterError, NotImplementedError

class PublicKey:
    default_attributes = {
        'id': None,     # int64 - Public key ID
        'title': None,     # string - Public key title
        'created_at': None,     # date-time - Public key created at date/time
        'fingerprint': None,     # string - Public key fingerprint
        'user_id': None,     # int64 - User ID.  Provide a value of `0` to operate the current session's user.
        'public_key': None,     # string - Actual contents of SSH key.
    }

    def __init__(self, attributes=None, options=None):
        if not isinstance(attributes, dict):
            attributes = {}
        if not isinstance(options, dict):
            options = {}
        self.set_attributes(attributes)
        self.options = options

    def set_attributes(self, attributes):
        for (attribute, default_value) in PublicKey.default_attributes.items():
            setattr(self, attribute, attributes.get(attribute, default_value))

    def get_attributes(self):
        return {k: getattr(self, k, None) for k in PublicKey.default_attributes if getattr(self, k, None) is not None}

    # Parameters:
    #   title (required) - string - Internal reference for key.
    def update(self, params = None):
        if not isinstance(params, dict):
            params = {}

        if hasattr(self, "id") and self.id:
            params['id'] = self.id
        else:
            raise MissingParameterError("Current object doesn't have a id")
        if "id" not in params:
            raise MissingParameterError("Parameter missing: id")
        if "title" not in params:
            raise MissingParameterError("Parameter missing: title")
        if "id" in params and not isinstance(params["id"], int):
            raise InvalidParameterError("Bad parameter: id must be an int")
        if "title" in params and not isinstance(params["title"], str):
            raise InvalidParameterError("Bad parameter: title must be an str")
        response, _options = Api.send_request("PATCH", "/public_keys/{id}".format(id=params['id']), params, self.options)
        return response.data

    def delete(self, params = None):
        if not isinstance(params, dict):
            params = {}

        if hasattr(self, "id") and self.id:
            params['id'] = self.id
        else:
            raise MissingParameterError("Current object doesn't have a id")
        if "id" not in params:
            raise MissingParameterError("Parameter missing: id")
        if "id" in params and not isinstance(params["id"], int):
            raise InvalidParameterError("Bad parameter: id must be an int")
        response, _options = Api.send_request("DELETE", "/public_keys/{id}".format(id=params['id']), params, self.options)
        return response.data

    def destroy(self, params = None):
        self.delete(params)

    def save(self):
        if hasattr(self, "id") and self.id:
            self.update(self.get_attributes())
        else:
            new_obj = create(self.get_attributes(), self.options)
            self.set_attributes(new_obj.get_attributes())

# Parameters:
#   user_id - int64 - User ID.  Provide a value of `0` to operate the current session's user.
#   cursor - string - Used for pagination.  Send a cursor value to resume an existing list from the point at which you left off.  Get a cursor from an existing list via the X-Files-Cursor-Next header.
#   per_page - int64 - Number of records to show per page.  (Max: 10,000, 1,000 or less is recommended).
def list(params = None, options = None):
    if not isinstance(params, dict):
        params = {}
    if not isinstance(options, dict):
        options = {}
    if "user_id" in params and not isinstance(params["user_id"], int):
        raise InvalidParameterError("Bad parameter: user_id must be an int")
    if "cursor" in params and not isinstance(params["cursor"], str):
        raise InvalidParameterError("Bad parameter: cursor must be an str")
    if "per_page" in params and not isinstance(params["per_page"], int):
        raise InvalidParameterError("Bad parameter: per_page must be an int")
    return ListObj(PublicKey,"GET", "/public_keys", params, options)

def all(params = None, options = None):
    list(params, options)

# Parameters:
#   id (required) - int64 - Public Key ID.
def find(id, params = None, options = None):
    if not isinstance(params, dict):
        params = {}
    if not isinstance(options, dict):
        options = {}
    params["id"] = id
    if "id" in params and not isinstance(params["id"], int):
        raise InvalidParameterError("Bad parameter: id must be an int")
    if "id" not in params:
        raise MissingParameterError("Parameter missing: id")
    response, options = Api.send_request("GET", "/public_keys/{id}".format(id=params['id']), params, options)
    return PublicKey(response.data, options)

def get(id, params = None, options = None):
    find(id, params, options)

# Parameters:
#   user_id - int64 - User ID.  Provide a value of `0` to operate the current session's user.
#   title (required) - string - Internal reference for key.
#   public_key (required) - string - Actual contents of SSH key.
def create(params = None, options = None):
    if not isinstance(params, dict):
        params = {}
    if not isinstance(options, dict):
        options = {}
    if "user_id" in params and not isinstance(params["user_id"], int):
        raise InvalidParameterError("Bad parameter: user_id must be an int")
    if "title" in params and not isinstance(params["title"], str):
        raise InvalidParameterError("Bad parameter: title must be an str")
    if "public_key" in params and not isinstance(params["public_key"], str):
        raise InvalidParameterError("Bad parameter: public_key must be an str")
    if "title" not in params:
        raise MissingParameterError("Parameter missing: title")
    if "public_key" not in params:
        raise MissingParameterError("Parameter missing: public_key")
    response, options = Api.send_request("POST", "/public_keys", params, options)
    return PublicKey(response.data, options)

# Parameters:
#   title (required) - string - Internal reference for key.
def update(id, params = None, options = None):
    if not isinstance(params, dict):
        params = {}
    if not isinstance(options, dict):
        options = {}
    params["id"] = id
    if "id" in params and not isinstance(params["id"], int):
        raise InvalidParameterError("Bad parameter: id must be an int")
    if "title" in params and not isinstance(params["title"], str):
        raise InvalidParameterError("Bad parameter: title must be an str")
    if "id" not in params:
        raise MissingParameterError("Parameter missing: id")
    if "title" not in params:
        raise MissingParameterError("Parameter missing: title")
    response, options = Api.send_request("PATCH", "/public_keys/{id}".format(id=params['id']), params, options)
    return PublicKey(response.data, options)

def delete(id, params = None, options = None):
    if not isinstance(params, dict):
        params = {}
    if not isinstance(options, dict):
        options = {}
    params["id"] = id
    if "id" in params and not isinstance(params["id"], int):
        raise InvalidParameterError("Bad parameter: id must be an int")
    if "id" not in params:
        raise MissingParameterError("Parameter missing: id")
    response, _options = Api.send_request("DELETE", "/public_keys/{id}".format(id=params['id']), params, options)
    return response.data

def destroy(id, params = None, options = None):
    delete(id, params, options)

def new(*args, **kwargs):
    return PublicKey(*args, **kwargs)


import datetime
from builtins import open as builtin_open
from datetime import datetime
import json
import io
from pathlib import Path
from . import file_action
from files_sdk.api import Api
from files_sdk.exceptions import InvalidParameterError, MissingParameterError, NotImplementedError

class File:
    default_attributes = {
        'path': None,     # string - File/Folder path This must be slash-delimited, but it must neither start nor end with a slash. Maximum of 5000 characters.
        'display_name': None,     # string - File/Folder display name
        'type': None,     # string - Type: `directory` or `file`.
        'size': None,     # int64 - File/Folder size
        'mtime': None,     # date-time - File last modified date/time, according to the server.  This is the timestamp of the last Files.com operation of the file, regardless of what modified timestamp was sent.
        'provided_mtime': None,     # date-time - File last modified date/time, according to the client who set it.  Files.com allows desktop, FTP, SFTP, and WebDAV clients to set modified at times.  This allows Desktop<->Cloud syncing to preserve modified at times.
        'crc32': None,     # string - File CRC32 checksum. This is sometimes delayed, so if you get a blank response, wait and try again.
        'md5': None,     # string - File MD5 checksum. This is sometimes delayed, so if you get a blank response, wait and try again.
        'mime_type': None,     # string - MIME Type.  This is determined by the filename extension and is not stored separately internally.
        'region': None,     # string - Region location
        'permissions': None,     # string - A short string representing the current user's permissions.  Can be `r`,`w`,`p`, or any combination
        'subfolders_locked?': None,     # boolean - Are subfolders locked and unable to be modified?
        'download_uri': None,     # string - Link to download file. Provided only in response to a download request.
        'priority_color': None,     # string - Bookmark/priority color of file/folder
        'preview_id': None,     # int64 - File preview ID
        'preview': None,     # File preview
        'action': None,     # string - The action to perform.  Can be `append`, `attachment`, `end`, `upload`, `put`, or may not exist
        'length': None,     # int64 - Length of file.
        'mkdir_parents': None,     # boolean - Create parent directories if they do not exist?
        'part': None,     # int64 - Part if uploading a part.
        'parts': None,     # int64 - How many parts to fetch?
        'ref': None,     # string -
        'restart': None,     # int64 - File byte offset to restart from.
        'structure': None,     # string - If copying folder, copy just the structure?
        'with_rename': None,     # boolean - Allow file rename instead of overwrite?
    }

    def __init__(self, *args):
        self.set_attributes({})
        self.options = {}
        self.mode = 'r'
        self.upload = None
        self.etags = None
        self.io_obj = io.StringIO()
        self.closed = True
        
        self.bytes_written = 0
        if len(args) >= 1:
            if isinstance(args[0], dict):
                self.set_attributes(args[0])
            elif isinstance(args[0], str):
                self.set_attributes({"path": args[0]})
        if len(args) >= 2:
            if isinstance(args[1], dict):
                self.options = args[1]
            elif isinstance(args[1], str):
                self.mode = args[1]
        if len(args) >= 3:
            if isinstance(args[2], dict):
                self.options = args[2]

    def set_attributes(self, attributes):
        for (attribute, default_value) in File.default_attributes.items():
            setattr(self, attribute, attributes.get(attribute, default_value))

    def get_attributes(self):
        return {k: getattr(self, k, None) for k in File.default_attributes if getattr(self, k, None) is not None}

    def __enter__(self):
        return self
    
    def __exit__(self, type, value, traceback):
        self.close()

    def __del__(self):
        if not self.io_obj.closed:
            self.io_obj.close()

    def close(self):
        self.flush()

        if self.upload:
            end_options = {
                "action": "end",
                "etags": self.etags,
                "provided_mtime": datetime.now().isoformat(),
                "ref": self.upload.ref,
                "size": self.bytes_written
            }

            file = create(self.path, end_options, self.options)
            self.set_attributes(file.get_attributes())
            self.mode = 'r'
            self.upload = None
            self.etags = None
            self.io_obj = io.StringIO()
        self.io_obj.close
        self.closed = True

    def fileno(self):
        raise OSError

    def flush(self, *_args):
        if "w" in self.mode:
            if self.io_obj.seekable():
                self.io_obj.seek(0)

            self.upload, self.etags, bytes_written = upload_chunks(self.io_obj, self.path, self.options, self.upload, self.etags)
            self.bytes_written += bytes_written
        elif "a" in self.mode:
            raise io.UnsupportedOperation("Append is not a supported mode")

    def isatty(self):
        return False

    def read(self):
        if self.readable():
            self.download_content(self.io_obj, False if "b" in self.mode else True)
            self.io_obj.seek(0)
            return self.io_obj.read()
        else:
            raise OSError("read mode not indicated")
    
    def readable(self):
        if 'r' in self.mode and not self.closed:
            return True
        return False
    
    def readall(self):
        self.read()
    
    def readinto(self):
        return NotImplementedError

    def readline(self):
        return NotImplementedError
    
    def readline(self):
        return NotImplementedError

    def readline(self):
        return NotImplementedError

    def seek(self):
        raise OSError

    def seekable(self):
        False

    def tell(self):
        raise OSError
    
    def truncate(self):
        raise OSError
    
    def writeable(self):
        if "w" in self.mode and not self.closed:
            return True
        return False

    def write(self, *args):
        if self.writeable():
            self.io_obj.write(*args)
        else:
            raise OSError("write mode not indicated")

    def download_uri_with_load(self):
        if self.download_uri:
            return self.download_uri 

        f = download(self.path, {}, self.options)
        self.set_attributes(f.get_attributes())
        return self.download_uri

    def download_content(self, io, is_string_io = False):
        Api.api_client().stream_download(self.download_uri_with_load(), io, is_string_io)

    def download_file(self, output_file):
        with builtin_open(output_file, 'wb') as file:
            self.download_content(file)

def open(path, mode = "r", options = None):
    if not isinstance(options, dict):
        options = {}
    file = File(path, mode, options)
    
    if "w" in mode:
        if "b" in mode:
            file.io_obj = io.BytesIO()
        else:
            file.io_obj = io.StringIO()

    if "r" in mode:
        if "b" in mode:
            file.io_obj = io.BytesIO()
        else:
            file.io_obj = io.StringIO()
    file.closed = False
    return file

def upload_chunks(io, path, options, upload = None, etags = None):
    if not etags:
        etags = []
    bytes_written = 0
    while True:
        headers = { "part": 1 } if not upload else { "ref": upload.ref, "part": upload.part_number + 1 }
        upload = file_action.begin_upload(path, headers, options)[0]
        buf = io.read(upload.partsize)
        if buf == b'' or buf == "":  # Empty bytearray means EOF for BytesIO, Empty String means EOF for StringIO
            return (upload, etags, bytes_written)
        if buf is not None:  # None means no data but io still open
            bytes_written += len(buf)
            response = Api.api_client().send_remote_request(upload.http_method, upload.upload_uri, { "Content-Length": str(len(buf)) }, buf)
            if "ETag" in response.headers:
                etags.append({ "etag": response.headers["ETag"].strip('"'), "part": upload.part_number })

def upload_file(path, destination = None, options = None):
    if not isinstance(options, dict):
        options = {}
    pth = Path(path)
    stat = pth.stat()
    with builtin_open(path, 'rb') as local_file:
        if destination is None:
            destination = pth.name
        upload, etags, _bytes_written = upload_chunks(local_file, destination, options)

        params = {
            "action": "end",
            "etags": etags,
            "provided_mtime": datetime.utcfromtimestamp(stat.st_mtime).isoformat(),
            "ref": upload.ref,
            "size": stat.st_size
        }

        create(destination, params, options)

def download_file(path, local_path = None, options = None):
    if not isinstance(options, dict):
        options = {}
    if local_path is None:
        local_path = Path(path).name
    return File(path, {}, options).download_file(local_path)

def find(path, params = None, options = None):
    if not isinstance(params, dict):
        params = {}
    if not isinstance(options, dict):
        options = {}
    params["action"] = "stat"
    return download(path, params, options)

    # Download file
    #
    # Parameters:
    #   action - string - Can be blank, `redirect` or `stat`.  If set to `stat`, we will return file information but without a download URL, and without logging a download.  If set to `redirect` we will serve a 302 redirect directly to the file.  This is used for integrations with Zapier, and is not recommended for most integrations.
    #   preview_size - string - Request a preview size.  Can be `small` (default), `large`, `xlarge`, or `pdf`.
    #   with_previews - boolean - Include file preview information?
    #   with_priority_color - boolean - Include file priority color information?
    def download(self, params = None):
        if not isinstance(params, dict):
            params = {}

        if hasattr(self, "path") and self.path:
            params['path'] = self.path
        else:
            raise MissingParameterError("Current object doesn't have a path")
        if "path" not in params:
            raise MissingParameterError("Parameter missing: path")
        if "path" in params and not isinstance(params["path"], str):
            raise InvalidParameterError("Bad parameter: path must be an str")
        if "action" in params and not isinstance(params["action"], str):
            raise InvalidParameterError("Bad parameter: action must be an str")
        if "preview_size" in params and not isinstance(params["preview_size"], str):
            raise InvalidParameterError("Bad parameter: preview_size must be an str")
        response, _options = Api.send_request("GET", "/files/{path}".format(path=params['path']), params, self.options)
        return response.data

    # Parameters:
    #   provided_mtime - string - Modified time of file.
    #   priority_color - string - Priority/Bookmark color of file.
    def update(self, params = None):
        if not isinstance(params, dict):
            params = {}

        if hasattr(self, "path") and self.path:
            params['path'] = self.path
        else:
            raise MissingParameterError("Current object doesn't have a path")
        if "path" not in params:
            raise MissingParameterError("Parameter missing: path")
        if "path" in params and not isinstance(params["path"], str):
            raise InvalidParameterError("Bad parameter: path must be an str")
        if "provided_mtime" in params and not isinstance(params["provided_mtime"], str):
            raise InvalidParameterError("Bad parameter: provided_mtime must be an str")
        if "priority_color" in params and not isinstance(params["priority_color"], str):
            raise InvalidParameterError("Bad parameter: priority_color must be an str")
        response, _options = Api.send_request("PATCH", "/files/{path}".format(path=params['path']), params, self.options)
        return response.data

    # Parameters:
    #   recursive - boolean - If true, will recursively delete folers.  Otherwise, will error on non-empty folders.
    def delete(self, params = None):
        if not isinstance(params, dict):
            params = {}

        if hasattr(self, "path") and self.path:
            params['path'] = self.path
        else:
            raise MissingParameterError("Current object doesn't have a path")
        if "path" not in params:
            raise MissingParameterError("Parameter missing: path")
        if "path" in params and not isinstance(params["path"], str):
            raise InvalidParameterError("Bad parameter: path must be an str")
        response, _options = Api.send_request("DELETE", "/files/{path}".format(path=params['path']), params, self.options)
        return response.data

    def destroy(self, params = None):
        self.delete(params)

    def save(self):
        new_obj = create(self.path, self.get_attributes(), self.options)
        self.set_attributes(new_obj.get_attributes())

# Download file
#
# Parameters:
#   action - string - Can be blank, `redirect` or `stat`.  If set to `stat`, we will return file information but without a download URL, and without logging a download.  If set to `redirect` we will serve a 302 redirect directly to the file.  This is used for integrations with Zapier, and is not recommended for most integrations.
#   preview_size - string - Request a preview size.  Can be `small` (default), `large`, `xlarge`, or `pdf`.
#   with_previews - boolean - Include file preview information?
#   with_priority_color - boolean - Include file priority color information?
def download(path, params = None, options = None):
    if not isinstance(params, dict):
        params = {}
    if not isinstance(options, dict):
        options = {}
    params["path"] = path
    if "path" in params and not isinstance(params["path"], str):
        raise InvalidParameterError("Bad parameter: path must be an str")
    if "action" in params and not isinstance(params["action"], str):
        raise InvalidParameterError("Bad parameter: action must be an str")
    if "preview_size" in params and not isinstance(params["preview_size"], str):
        raise InvalidParameterError("Bad parameter: preview_size must be an str")
    if "path" not in params:
        raise MissingParameterError("Parameter missing: path")
    response, options = Api.send_request("GET", "/files/{path}".format(path=params['path']), params, options)
    return File(response.data, options)

# Parameters:
#   path (required) - string - Path to operate on.
#   action - string - The action to perform.  Can be `append`, `attachment`, `end`, `upload`, `put`, or may not exist
#   etags[etag] (required) - array(string) - etag identifier.
#   etags[part] (required) - array(int64) - Part number.
#   length - int64 - Length of file.
#   mkdir_parents - boolean - Create parent directories if they do not exist?
#   part - int64 - Part if uploading a part.
#   parts - int64 - How many parts to fetch?
#   provided_mtime - string - User provided modification time.
#   ref - string -
#   restart - int64 - File byte offset to restart from.
#   size - int64 - Size of file.
#   structure - string - If copying folder, copy just the structure?
#   with_rename - boolean - Allow file rename instead of overwrite?
def create(path, params = None, options = None):
    if not isinstance(params, dict):
        params = {}
    if not isinstance(options, dict):
        options = {}
    params["path"] = path
    if "path" in params and not isinstance(params["path"], str):
        raise InvalidParameterError("Bad parameter: path must be an str")
    if "action" in params and not isinstance(params["action"], str):
        raise InvalidParameterError("Bad parameter: action must be an str")
    if "length" in params and not isinstance(params["length"], int):
        raise InvalidParameterError("Bad parameter: length must be an int")
    if "part" in params and not isinstance(params["part"], int):
        raise InvalidParameterError("Bad parameter: part must be an int")
    if "parts" in params and not isinstance(params["parts"], int):
        raise InvalidParameterError("Bad parameter: parts must be an int")
    if "provided_mtime" in params and not isinstance(params["provided_mtime"], str):
        raise InvalidParameterError("Bad parameter: provided_mtime must be an str")
    if "ref" in params and not isinstance(params["ref"], str):
        raise InvalidParameterError("Bad parameter: ref must be an str")
    if "restart" in params and not isinstance(params["restart"], int):
        raise InvalidParameterError("Bad parameter: restart must be an int")
    if "size" in params and not isinstance(params["size"], int):
        raise InvalidParameterError("Bad parameter: size must be an int")
    if "structure" in params and not isinstance(params["structure"], str):
        raise InvalidParameterError("Bad parameter: structure must be an str")
    if "path" not in params:
        raise MissingParameterError("Parameter missing: path")
    response, options = Api.send_request("POST", "/files/{path}".format(path=params['path']), params, options)
    return File(response.data, options)

# Parameters:
#   provided_mtime - string - Modified time of file.
#   priority_color - string - Priority/Bookmark color of file.
def update(path, params = None, options = None):
    if not isinstance(params, dict):
        params = {}
    if not isinstance(options, dict):
        options = {}
    params["path"] = path
    if "path" in params and not isinstance(params["path"], str):
        raise InvalidParameterError("Bad parameter: path must be an str")
    if "provided_mtime" in params and not isinstance(params["provided_mtime"], str):
        raise InvalidParameterError("Bad parameter: provided_mtime must be an str")
    if "priority_color" in params and not isinstance(params["priority_color"], str):
        raise InvalidParameterError("Bad parameter: priority_color must be an str")
    if "path" not in params:
        raise MissingParameterError("Parameter missing: path")
    response, options = Api.send_request("PATCH", "/files/{path}".format(path=params['path']), params, options)
    return File(response.data, options)

# Parameters:
#   recursive - boolean - If true, will recursively delete folers.  Otherwise, will error on non-empty folders.
def delete(path, params = None, options = None):
    if not isinstance(params, dict):
        params = {}
    if not isinstance(options, dict):
        options = {}
    params["path"] = path
    if "path" in params and not isinstance(params["path"], str):
        raise InvalidParameterError("Bad parameter: path must be an str")
    if "path" not in params:
        raise MissingParameterError("Parameter missing: path")
    response, _options = Api.send_request("DELETE", "/files/{path}".format(path=params['path']), params, options)
    return response.data

def destroy(path, params = None, options = None):
    delete(path, params, options)

def new(*args, **kwargs):
    return File(*args, **kwargs)